//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "ScriptExec.h"
#include "LuaState.h"
#include "lua.hpp"
#include <Clipbrd.hpp>
#include <string>
#include <assert.h>
#include <time.h>

namespace {
	// ugly "global" context
	ScriptExec *context = NULL;

	long timediff(clock_t t1, clock_t t2) {
		long elapsed;
		elapsed = static_cast<long>(((double)t2 - t1) / CLOCKS_PER_SEC * 1000);
		return elapsed;
	}
}

int ScriptExec::LuaPrint(lua_State *L)
{
	int nArgs = lua_gettop(L);
	int i;
	lua_getglobal(L, "tostring");
	std::string ret;//this is where we will dump the output
	//make sure you start at 1 *NOT* 0
	for(i = 1; i <= nArgs; i++)
	{
		const char *s;
		lua_pushvalue(L, -1);
		lua_pushvalue(L, i);
		lua_call(L, 1, 1);
		s = lua_tostring(L, -1);
		if(s == NULL)
			return luaL_error(L, LUA_QL("tostring") " must return a string to ", LUA_QL("print"));

		if(i > 1) ret.append("\t");

		ret.append(s);
		lua_pop(L, 1);
	};
	//ret.append("\n");
	context->onAddOutputText(ret.c_str());
	return 0;
};

int ScriptExec::LuaError( lua_State *L )
{
	const char* str = lua_tostring( L, -1 );
	lua_pop(L, 1);
	//send string to console
	context->onAddOutputText(str);
	return 0;
}

int ScriptExec::l_ShowMessage( lua_State* L )
{
	AnsiString msg = lua_tostring(L, -1);
	MessageBox(NULL, msg.c_str(), "Lua message", MB_ICONINFORMATION);
	lua_pushnumber( L, 0 );
	return 1;
}

int ScriptExec::l_Sleep(lua_State* L)
{
	int ret = 0;
	int time = lua_tounsigned(L, -1);
	clock_t t1 = clock();
	long elapsed = 0;
	if (context->breakReq)
	{
		ret = -1;
	}
	else
	{
		while (elapsed < time)
		{
			if (context->breakReq)
			{
				ret = -1;
				break;
			}
			Application->ProcessMessages();
			Sleep(1);
			elapsed = timediff(t1, clock());
			if (context->breakReq)
			{
				ret = -1;
				break;
			}
		}
	}
	lua_pushnumber(L, ret);
	return 1;					// number of return values
}

/** Beep(frequency, time_ms)
	\note blocking and not interruptable
*/
int ScriptExec::l_Beep(lua_State *L)
{
	/*
	To refer to elements in the stack, the API uses indices. The first element in the stack (that is, the element that was pushed first) has index 1, the next one has index 2, and so on. We can also access elements using the top of the stack as our reference, using negative indices. In that case, -1 refers to the element at the top (that is, the last element pushed), -2 to the previous element, and so on. For instance, the call lua_tostring(L, -1) returns the value at the top of the stack as a string.
	*/
	int freq = lua_tounsigned(L, 1);
	int time = lua_tounsigned(L, 2);
	Beep(freq, time);
	return 0;
}

int ScriptExec::l_GetClipboardText( lua_State* L )
{
	AnsiString text = Clipboard()->AsText;
	lua_pushstring( L, text.c_str() );
	return 1;
}

int ScriptExec::l_SetClipboardText( lua_State* L )
{
	const char* str = lua_tostring( L, -1 );
	Clipboard()->AsText = str;
	return 0;
}

int ScriptExec::l_Call(lua_State* L)
{
	const char* str = lua_tostring( L, -1 );
	context->onCall(str);
	return 0;
}

int ScriptExec::l_Hangup(lua_State* L)
{
	context->onHangup();
	return 0;
}

int ScriptExec::l_SetDial(lua_State* L)
{
	const char* str = lua_tostring( L, -1 );
	context->onSetDial(str);
	return 0;
}

int ScriptExec::l_GetDial(lua_State* L)
{
	std::string num = context->onGetDial();
	lua_pushstring( L, num.c_str() );
	return 1;
}

ScriptExec::ScriptExec(
	CallbackAddOutputText onAddOutputText,
	CallbackCall onCall,
	CallbackHangup onHangup,
	CallbackGetDial onGetDial,
	CallbackSetDial onSetDial
	):
	onAddOutputText(onAddOutputText),
	onCall(onCall),
	onHangup(onHangup),
	onGetDial(onGetDial),
	onSetDial(onSetDial),
	breakReq(false),
	running(false)
{
	assert(onAddOutputText && onCall && onHangup && onGetDial && onSetDial);
}

ScriptExec::~ScriptExec()
{
}

void ScriptExec::Run(const char* script)
{
	breakReq = false;
	running = true;
	context = this;
	Lua_State L;
	luaL_openlibs(L);

	lua_register(L, "_ALERT", LuaError );
	lua_register(L, "print", LuaPrint );
	lua_register(L, "ShowMessage", l_ShowMessage);
	lua_register(L, "Beep", l_Beep);
	lua_register(L, "GetClipboardText", l_GetClipboardText);
	lua_register(L, "SetClipboardText", l_SetClipboardText);
	lua_register(L, "Call", l_Call);
	lua_register(L, "Hangup", l_Hangup);
	lua_register(L, "GetDial", l_GetDial);
	lua_register(L, "SetDial", l_SetDial);

	int res = luaL_dostring(L, script);
	if(res != 0)
	{
		AnsiString txt;
		txt.sprintf("Execution error:\n%s", lua_tostring(L, -1));
		MessageBox(NULL, txt.c_str(), "Lua", MB_ICONINFORMATION);
	}
	running = false;
}

void ScriptExec::Break(void)
{
	breakReq = true;
}

